"""Run error analysis on an out file."""
import re
import sys

def read(filename):
  cur_split = None
  train_ex = []
  dev_ex = []
  cur_ex = {}
  with open(filename) as f:
    for line in f:
      line = line.strip()
      if line == 'Training data:':
        cur_split = 'train'
      elif line == 'Dev data:':
        cur_split = 'dev'
      if not cur_split: continue
      me = re.match('Example ([0-9]+)', line)
      mx = re.match('x      = "(.*)"', line)
      my = re.match('y      = "(.*)"', line)
      my_pred = re.match('y_pred = "(.*)"', line)
      if me:
        cur_ex = {}
        if cur_split == 'train':
          train_ex.append(cur_ex)
        else:
          dev_ex.append(cur_ex)
        cur_ex['num'] = int(me.group(1))
      elif mx:
        cur_ex['x'] = mx.group(1)
      elif my:
        cur_ex['y'] = my.group(1)
      elif my_pred:
        cur_ex['y_pred'] = my_pred.group(1)
      elif line.startswith('denotation correct'):
        cur_ex['denotation_correct'] = (line.split(' ')[-1] == 'True')
  return train_ex, dev_ex

def run_analysis(examples):
  num_error = 0
  num_drop = 0
  num_same_len = 0
  num_extra = 0
  for ex in examples:
    if ex['denotation_correct']: continue
    num_error += 1
    y_words = [w for w in ex['y'].split(' ') if len(w) > 1]
    y_pred_words = [w for w in ex['y_pred'].split(' ') if len(w) > 1]
    if len(y_pred_words) < len(y_words):
      #print y_words, y_pred_words
      num_drop += 1
    elif len(y_pred_words) > len(y_words):
      num_extra += 1
    else:
      num_same_len += 1
  print 'Num drop: %d' % num_drop
  print 'Num extra: %d' % num_extra
  print 'Num same len: %d' % num_same_len
  print 'Num error: %d' % num_error
  print 'Total: %d' % len(examples)

def run(*filenames):
  for filename in filenames:
    print 'Filename: %s' % filename
    train_ex, dev_ex = read(filename)
    run_analysis(dev_ex)

def main():
  if len(sys.argv) < 2:
    print >> sys.stderr, 'Usage: %s [filename list]' % sys.argv[0]
    sys.exit(1)
  run(*sys.argv[1:])

if __name__ == '__main__':
  main()
